@@ -7,7 +7,7 @@
 # https://github.com/manatlan/reqman4
 # #############################################################################
 
-__version__ = r"0.3.0" # <-- replaced by build.py (extract from pyproject.toml)
+__version__ = r"0.5.0" # <-- replaced by build.py (extract from pyproject.toml)
 
 """
 Special keys :
@@ -15,7 +15,7 @@ Special keys :
  * proxy: <string>          # default None
  * timeout: <milliseconds>  # default 60_000 ms
  * root: <string>           # default None
- * switch: <dict/mapping>   # default {}
+ * --<switch>: <dict/mapping>   # default {}
  * headers: <dict/mapping>   # default {}
  * BEGIN: <list>
  * END: <list>
