@@ -13,6 +13,10 @@ import logging
 import traceback
 import tempfile
 import webbrowser
+from itertools import chain 
+import glob
+
+# pypi packages
 import click
 from colorama import init, Fore, Style
 from urllib.parse import unquote
@@ -39,8 +43,8 @@ cw = lambda t: colorize(Fore.WHITE, t)
 
 
 class Output:
-    def __init__(self,switch:str|None):
-        self.switch = switch
+    def __init__(self,switchs:tuple):
+        self.switchs = switchs
         self.nb_tests=0
         self.nb_tests_ok=0
         self.nb_req=0
@@ -78,7 +82,7 @@ class Output:
         pass
 
     def end_tests(self):
-        self.htmls.append( output.generate_final( self.switch, self.nb_tests_ok, self.nb_tests) )
+        self.htmls.append( output.generate_final( self.switchs, self.nb_tests_ok, self.nb_tests) )
 
         if self.error:
             print(cr(f"SCENARIO ERROR: {self.error}"))
@@ -99,69 +103,57 @@ class Output:
         webbrowser.open(f'file://{os.path.abspath(temp_html_path)}')        
 
 
-def display_env( x ):
-    print(cy("Final environment:"))
-    print(x if x is not None else "no env")
-
-
 
 class ExecutionTests:
-    def __init__(self,files:list,switch:str|None=None,vars:dict={},is_debug=False):
-        # fix files : extract files (yml/rml) from potentials directories
+    def __init__(self,files:list,vars:dict={},is_debug=False, compatibility:int=0):
         self.files=common.expand_files(files)
         self.is_debug=is_debug
-        
+        self.env=None
+
         # init the conf
         reqman_conf = common.guess_reqman_conf(self.files)
         if reqman_conf is None:
-            conf = {}
+            self.conf_global = common.Conf({})
         else:
             print(cy(f"Using {os.path.relpath(reqman_conf)}"))
-            conf = common.load_reqman_conf(reqman_conf)
+            self.conf_global = common.Conf(common.load_reqman_conf(reqman_conf))
+
+        # init with the switchs from conf
+        self.switchs = self.conf_global.switchs
 
         # update with vars from command line
-        conf.update(vars)
-
-        self.env = env.Env( **conf )
-
-        # apply the switch
-        if switch:
-            # First, load all scenarios to get all possible switches
-            for file in self.files:
-                # just load to get switches in self.env
-                scenario.Scenario(file, self.env)
-
-            common.assert_syntax(switch in self.env.switchs.keys(), f"Unknown switch '{switch}'")
-            self.env.update( self.env.switchs[switch] )
-        self._switch = switch
+        self.conf_global.update(vars)
 
 
-    def view(self):
+        self.scenarios=[]
+        # merge the switchs from others files
         for f in self.files:
-            print(cb(f"Analyse {f}"))
-            s=scenario.Scenario(f, self.env)
+            s=scenario.Scenario(f, compatibility)
+            self.switchs.update( s._ys.conf.switchs )
+            self.scenarios.append(s)
 
-            if "BEGIN" in self.env:
-                print("BEGIN", scenario.StepCall(s, {scenario.OP.CALL:"BEGIN"}) )
 
-            for i in s:
-                print(i)
 
-            if "END" in self.env:
-                print("END", scenario.StepCall(s, {scenario.OP.CALL:"END"}) )
+    async def execute(self,*switchs) -> Output:
+        """ Run all tests in files """
 
-    async def execute(self) -> Output:
-        """ Run all tests in files, return number of failed tests """
-        output = Output(self._switch)
+        # apply switchs for global conf (aka reqman_conf)
+        self.conf_global.apply(*switchs)
 
-        for file in self.files:
-            output.begin_scenario(file)
+        # create the real env
+        self.env = env.Env( **self.conf_global )
 
+        output = Output(switchs)
+
+        for idx,scenar in enumerate(self.scenarios):
+            output.begin_scenario(scenar.file_path)
+
+            # update conf of scenar into env
+            self.env.update( scenar.conf.apply(*switchs) )
+    
             try:
-                scenar = scenario.Scenario(file, self.env)
-                async for req in scenar.execute(with_begin=(file == self.files[0]), with_end=(file == self.files[-1])):
+                async for req in scenar.execute( self.env, with_begin=(idx==0), with_end=(idx==len(self.scenarios)) ):
                     output.write_a_test(req)
-                self.env = scenar.env  # needed !
             except common.RqException as ex:
                 if self.is_debug:
                     traceback.print_exc()
@@ -178,45 +170,7 @@ class ExecutionTests:
 
 
 
-def guess(args:list):
-    ##########################################################################
-    files = common.expand_files([i for i in args if os.path.exists(i)])
-    reqman_conf = common.guess_reqman_conf(files)
-    if reqman_conf:
-        conf = common.load_reqman_conf(reqman_conf)
-    else:
-        conf = {}
-
-    if len(files)==1:
-        # an unique file
-        s = scenario.Scenario(files[0],env.Env(**conf))
-        if s.env.switchs:
-            print(cy(f"Using switches from {files[0]}"))
-        return s.env.switchs
-    else:
-        return env.Env(**conf).switchs
-    ##########################################################################
-
-def options_from_files(opt_name:str):
-    try:
-        d=guess(sys.argv[1:] or [])
-    except common.RqException as ex:
-        print(cr(f"START ERROR: {ex}"))
-        sys.exit(-1)
-
-    ll=[dict( name=k, switch=f"--{k}", help=v.get("doc","???") ) for k,v in d.items()]
-
-    def decorator(f):
-        for p in reversed(ll):
-            click.option( p['switch'], opt_name, 
-                is_flag = True,
-                flag_value=p['name'],
-                required = False,
-                help = p['help'],
-            )(f)
-        return f
-    return decorator
-
+#- ----------------------------------------------------------
 
 @click.group()
 def cli():
@@ -227,23 +181,40 @@ def patch_docstring(f):
     f.__doc__+= f" (version:{VERSION})"
     return f
 
-@cli.command()
+@cli.command(context_settings=dict(allow_extra_args=True, ignore_unknown_options=True))
 @click.argument('files', nargs=-1, required=True ) #help="Scenarios yml/rml (local or http)"
-# @click.argument('files', type=click.Path(exists=True,), nargs=-1, required=True)
-@options_from_files("switch")
-@click.option('-v',"is_view",is_flag=True,default=False,help="Analyze only, do not execute requests")
 @click.option('-d',"is_debug",is_flag=True,default=False,help="debug mode")
 @click.option('-e',"show_env",is_flag=True,default=False,help="Display final environment")
 @click.option('-s',"vars",help="Set variables (ex: -s token=DEADBEAF,id=42)")
 @click.option('-i',"is_shebang",is_flag=True,default=False,help="interactif mode (with shebang)")
 @click.option('-o',"open_browser",is_flag=True,default=False,help="open result in an html page")
+@click.option('-c',"compatibility",is_flag=True,default=False,help="accept old reqman3 scenarios")
+@click.option('-cc',"comp_convert",is_flag=True,default=False,help="accept old reqman3 and generate new version")
+@click.option("-h",'--help',"need_help",is_flag=True,default=False,help="to get help")
+@click.pass_context
+
+
 @patch_docstring
-def command(**p) -> int:
+def command(ctx,**p):
     """Test an http service with pre-made scenarios, whose are simple yaml files
 (More info on https://github.com/manatlan/reqman4) """
-    return reqman(**p)
+    files = [f for f in p["files"] if not f.startswith("--")]
+    p["switchs"] = [f[2:] for f in p["files"] if f.startswith("--")]
+    p["files"] = files
+    sys.exit( reqman(ctx,**p) )
+
+def reqman(ctx, files:list,vars:str="",show_env:bool=False,is_debug:bool=False,is_shebang:bool=False,open_browser:bool=False,compatibility:bool=False,comp_convert:bool=False,need_help:bool=False,switchs:list=[]) -> int:
+
+
+    files = list(chain.from_iterable([glob.glob(i,recursive=True) for i in files]))
+
+    if compatibility:
+        comp_mode=1
+    elif comp_convert:
+        comp_mode=2
+    else:
+        comp_mode=0
 
-def reqman(files:list,switch:str|None=None,vars:str="",show_env:bool=False,is_debug:bool=False,is_view:bool=False,is_shebang:bool=False,open_browser:bool=False) -> int:
     if vars:
         dvars = dict( [ i.split("=",1) for i in vars.split(",") if "=" in i ] )
     else:
@@ -267,25 +238,27 @@ def reqman(files:list,switch:str|None=None,vars:str="",show_env:bool=False,is_de
         logging.basicConfig(level=logging.ERROR)
 
     try:
-        r = ExecutionTests( files,switch,dvars, is_debug)
-        if is_view:
-            r.view()
+        r = ExecutionTests( files,dvars, is_debug, comp_mode)
+        if need_help:
+            click.echo(ctx.get_help())
+            for k,v in r.switchs.items():
+                click.echo(f"  --{k}      {v.get('doc','??')}")
             return 0
+        o = asyncio.run(r.execute(*switchs))
+
+        if show_env:
+            print(cy("Final environment:"))
+            print(r.env)
+
+        if o.error:
+            rc = -1
         else:
-            o = asyncio.run(r.execute())
+            rc = o.nb_tests_ko
 
-            if show_env:
-                display_env(r.env)
+        if open_browser:
+            o.open_browser()
 
-            if o.error:
-                rc = -1
-            else:
-                rc = o.nb_tests_ko
-
-            if open_browser:
-                o.open_browser()
-
-            return rc
+        return rc
 
     except Exception as ex:
         # everything that happen here is an real bug/error
