@@ -6,7 +6,8 @@
 #
 # https://github.com/manatlan/reqman4
 # #############################################################################
-import yaml,os,time
+import os
+import time
 import httpx
 from typing import Any, AsyncGenerator
 
@@ -16,9 +17,6 @@ from .common import assert_syntax
 from . import env
 from . import ehttp
 
-from . import compat
-FIX_SCENAR = compat.fix_scenar
-FIX_TESTS = compat.fix_tests
 
 import logging
 logger = logging.getLogger(__name__)
@@ -29,6 +27,7 @@ class OP:
 
 class Step:
     params: list|str|None = None
+    line: int|None = None
     
     async def process(self,e:env.Env) -> AsyncGenerator:
         ...
@@ -47,22 +46,23 @@ class Step:
 
 
 class StepCall(Step):
-    def __init__(self, scenario: "Scenario", step: dict, params:list|str|None=None):
+    def __init__(self, scenario: "Scenario", step: dict, env:env.Env, params:list|str|None=None, line:int|None=None):
+        self.line = line
         self.scenario = scenario
         self.params = params
         self.steps=[]
 
         # extract step into local properties
-        name = step[OP.CALL]
+        self.name = step[OP.CALL]
 
         assert_syntax( len(step.keys()) == 1, f"unknowns call'attributes: {list(step.keys())}")
-        assert_syntax( isinstance(name, str),"CALL must be a string")
-        assert_syntax( name in self.scenario.env,f"CALL references unknown scenario '{name}'")
+        assert_syntax( isinstance(self.name, str),"CALL must be a string")
+        assert_syntax( self.name in env,f"CALL references unknown scenario '{self.name}'")
         
-        sub_scenar = self.scenario.env[name]
+        sub_scenar = env[self.name]
         assert_syntax( isinstance(sub_scenar, list),"CALL must reference a list of steps")
 
-        self.steps = self.scenario._feed( sub_scenar )
+        self.steps = self.scenario._feed( env, sub_scenar )
 
     async def process(self,e:env.Env) -> AsyncGenerator:
 
@@ -82,14 +82,15 @@ class StepCall(Step):
         for i in self.steps:
             s+= "  - "+repr(i)+"\n"
         if self.params:
-            return f"CALL MULTIPLE with {self.params}:\n"+s
+            return f"Step CALL:{self.name} with PARAMS:{self.params}:\n"+s
         else:
-            return f"CALL:\n"+s
+            return f"Step CALL:{self.name}:\n"+s
 
 
 
 class StepHttp(Step):
-    def __init__(self, scenario: "Scenario", step: dict, params: list|str|None=None):
+    def __init__(self, scenario: "Scenario", step: dict, params: list|str|None=None, line:int|None=None):
+        self.line = line
         self.scenario = scenario
         self.params = params
 
@@ -106,54 +107,54 @@ class StepHttp(Step):
         self.doc = step.get("doc","")
         self.headers = step.get("headers",{})
         self.body = step.get("body",None)
-        self.tests = FIX_TESTS( step.get("tests",[]) )
+        self.tests = step.get("tests",[])
 
         assert_syntax(isinstance(self.tests,list),"tests must be a list of strings")
         assert_syntax(all( isinstance(t,str) for t in self.tests ),"tests must be a list of strings")
 
 
-    def _prepare_request(self, e: env.Env) -> tuple:
-        url = e.substitute(self.url)
-        root = e.get("root", "")
+    def _prepare_request(self, env: env.Env) -> tuple:
+        url = env.substitute(self.url)
+        root = env.get("root", "")
         if root and url.startswith("/"):
             url = root + url
         assert_syntax(url.startswith("http"), f"url must start with http, found {url}")
 
-        headers = self.scenario.env.get("headers", {}) or {}
+        headers = env.get("headers", {}) or {}
         headers.update(self.headers)
-        headers = e.substitute_in_object(headers)
+        headers = env.substitute_in_object(headers)
         # httpx requires header values to be strings, so convert them all
         headers = {k: str(v) for k, v in headers.items()}
 
         body = self.body
         if body:
             if isinstance(body, str):
-                body = e.substitute(body)
+                body = env.substitute(body)
             elif isinstance(body, (dict, list)):
-                body = e.substitute_in_object(body)
+                body = env.substitute_in_object(body)
 
         return url, headers, body
 
-    async def _execute_request(self, e: env.Env, url: str, headers: dict, body: Any) -> httpx.Response:
-        print(f"Executing request: method={self.method}, url={url}, headers={headers}, body={body}")
+    async def _execute_request(self, env: env.Env, url: str, headers: dict, body: Any) -> httpx.Response:
+        #print(f"Executing request: method={self.method}, url={url}, headers={headers}, body={body}")
         start = time.time()
         response = await ehttp.call(
             self.method,
             url,
             body,
             headers=httpx.Headers(headers),
-            proxy=e.get("proxy", None),
-            timeout=e.get("timeout", 60_000) or 60_000,  # 60 sec
+            proxy=env.get("proxy", None),
+            timeout=env.get("timeout", 60_000) or 60_000,  # 60 sec
         )
         diff_ms = round((time.time() - start) * 1000)
-        e.set_R_response(response, diff_ms)
+        env.set_R_response(response, diff_ms)
         return response
 
-    def _process_response(self, e: env.Env, response: httpx.Response) -> common.Result:
+    def _process_response(self, enw: env.Env, response: httpx.Response) -> common.Result:
         results = []
         for t in self.tests:
             try:
-                ok, dico = e.eval(t, with_context=True)
+                ok, dico = enw.eval(t, with_context=True)
                 context = "Variables:\n"
                 for k, v in dico.items():
                     context += f"  {k}: {env.jzon_dumps(v, indent=None)}\n"
@@ -162,7 +163,7 @@ class StepHttp(Step):
                 logger.error(f"Can't eval test [{t}] : {ex}")
                 results.append(common.TestResult(None, t, f"ERROR: {ex}"))
 
-        doc = e.substitute(self.doc, raise_error=False)
+        doc = enw.substitute(self.doc, raise_error=False)
         return common.Result(response.request, response, results, doc=doc)
 
     async def process(self, e: env.Env) -> AsyncGenerator:
@@ -177,16 +178,16 @@ class StepHttp(Step):
 
             e.scope_revert(param)
 
-    
 
     def __repr__(self):
         if self.params:
-            return f"HTTP MULTIPLE {self.method} {self.url} with {self.params}"
+            return f"Step {self.method}:{self.url} with PARAMS:{self.params}"
         else:   
-            return f"HTTP {self.method} {self.url}"
+            return f"Step {self.method}:{self.url}"
 
 class StepSet(Step):
-    def __init__(self, scenario: "Scenario", step:dict):
+    def __init__(self, scenario: "Scenario", step:dict, line:int|None=None):
+        self.line = line
         self.scenario = scenario
 
         assert_syntax( len(step) == 1,"SET cannot be used with other keys")
@@ -195,22 +196,16 @@ class StepSet(Step):
         self.dico = dico
 
     async def process(self,e:env.Env) -> AsyncGenerator:
-        e.update( e.substitute_in_object(self.dico,raise_error=False) )
+        e.update( e.substitute_in_object(self.dico,raise_error=True) )
         yield None
 
     def __repr__(self):
-        return f"SET {self.dico}"
+        return f"Step SET {self.dico}"
 
 
 
 class Scenario(list):
-    def __init__(self, file_path: str, e:env.Env|None=None):
-        if e:
-            assert isinstance(e,env.Env)
-        else:
-            e=env.Env()
-
-        self.env = e
+    def __init__(self, file_path: str, is_compatibility:int=0):
 
         if file_path.startswith("http"):
             try:
@@ -218,9 +213,9 @@ class Scenario(list):
             except Exception as ex:
                 raise common.RqException(f"[URI:{file_path}] [http error] [{ex}]")
         else:
+            file_path = os.path.relpath(file_path)
             if os.path.isfile(file_path):
-                with open(file_path, 'r') as fid:
-                    yml_str = fid.read()
+                yml_str = open(file_path, 'r')
             else:
                 raise common.RqException(f"[{file_path}] [File not found]")
         self.file_path = file_path
@@ -228,23 +223,27 @@ class Scenario(list):
         list.__init__(self,[])
 
         try:
-            conf,scenar = common.load_scenar(yml_str)
-            conf,scenar = FIX_SCENAR( conf, scenar)
-        except yaml.YAMLError as ex:
+            self._ys = common.YScenario(yml_str,is_compatibility)
+        except Exception as ex:
             raise common.RqException(f"[{file_path}] [Bad syntax] [{ex}]")
 
-        self.env.update( conf ) # this override a reqman.conf env !
-        self.extend( self._feed( scenar ) )
+    @property
+    def conf(self) -> common.Conf:
+        return self._ys.conf
 
+    @property
+    def steps(self) -> list:
+        return self._ys._steps
 
-    def _feed(self, liste:list[dict]) -> list[Step]:
+    def _feed(self, env:env.Env, liste:list[dict]) -> list[Step]:
         try:
             step=None
             assert_syntax(isinstance(liste, list),"RUN must be a list")
 
             ll = []
             for step in liste:
-                assert_syntax( isinstance(step, dict), f"Bad step {step}")
+                line = step.lc.line if hasattr(step, 'lc') else None
+                assert_syntax( isinstance(step, dict), f"Bad Dict {step}")
                 
                 if "params" in step:
                     params=step["params"]
@@ -254,41 +253,50 @@ class Scenario(list):
 
                 if OP.SET in step:
                     assert_syntax( params is None, "params cannot be used with set")
-                    ll.append( StepSet( self, step ) )
+                    ll.append( StepSet( self, step, line ) )
                 else:
                     if OP.CALL in step:
-                        ll.append( StepCall( self, step, params ) )
+                        ll.append( StepCall( self, step, env, params, line ) )
                     else:
                         if set(step.keys()) & ehttp.KNOWNVERBS:
-                            ll.append( StepHttp( self, step, params ) )
+                            ll.append( StepHttp( self, step, params, line ) )
                         else:
-                            raise common.RqException(f"Bad step {step}")
+                            raise common.RqException(f"Bad Dict {step}")
             return ll
         except common.RqException as ex:
-            raise common.RqException(f"[{self.file_path}] [Bad step {step}] [{ex}]")
+            # this is an ERROR at compilation time
+            line_info = f":{step.lc.line+1}" if hasattr(step, 'lc') and step.lc else ""
+            raise common.RqException(f"[{self.file_path}{line_info}] [Bad {step}] [{ex}]")
     
     def __repr__(self):
         return super().__repr__()
     
-    async def execute(self,with_begin:bool=False,with_end:bool=False) -> AsyncGenerator:
+    async def execute(self,env,with_begin:bool=False,with_end:bool=False) -> AsyncGenerator:
+        self.clear()
+        self.extend( self._feed( env, self.steps ) )
+
+        step=None
         try:
 
-            if with_begin and self.env.get("BEGIN"):
+            if with_begin and env.get("BEGIN"):
                 logger.debug("Execute BEGIN statement")
-                async for i in StepCall(self, {OP.CALL:"BEGIN"}).process(self.env):
+                async for i in StepCall(self, {OP.CALL:"BEGIN"}, env).process(env):
                     yield i
 
             for step in self:
-                async for i in step.process(self.env):
+                logger.debug("Execute STEP %s",step)
+                async for i in step.process(env):
                     yield i
 
-            if with_end and self.env.get("END"):
+            if with_end and env.get("END"):
                 logger.debug("Execute END statement")
-                async for i in StepCall(self, {OP.CALL:"END"} ).process(self.env):
+                async for i in StepCall(self, {OP.CALL:"END"}, env ).process(env):
                     yield i
 
         except Exception as ex:
-            raise common.RqException(f"[{self.file_path}] [Error Step {step}] [{ex}]")
+            # this is an ERROR at execution time
+            line_info = f":{step.line+1}" if hasattr(step, 'line') and step.line else ""
+            raise common.RqException(f"[{self.file_path}{line_info}] [Error {step}] [{ex}]")
 
 
 
